@@ -178,7 +178,7 @@ sub _build_tree {
   my $self = shift;
 
   my $tree = $self->tree;
-  foreach my $id ( sort keys %$tree ) {
+  foreach my $id ( sort { $a <=> $b } keys %$tree ) {
       my $node        = $tree->{$id};
       $node->{id}     = $id;
       my $parent      = $tree->{$node->{parent}};
@@ -233,16 +233,21 @@ sub  _bckd_update {
 sub _check_rules{
     my ($self, $node, $parent) = @_;
 
+    $node->{id} = $self->_last_node->{id} +1
+        if ( ! defined $node->{id} );
+
     return 1 if (  ! defined $self->config ||
                    ! defined $self->config->{rules} ||
                    $node->{id} == 1
                 );
-
     # Is node can 'link' to parent ?
     my $rules = $self->_rules;
-    if ( defined $rules->{$node->{$self->_type_field}}) {
-        if ( defined $rules->{$node->{$self->_type_field}}->{$parent->{$self->_type_field}}) {
-            return $rules->{$node->{$self->_type_field}}->{$parent->{$self->_type_field}};
+    if ( defined $node->{$self->_type_field} ) {
+        if ( defined $rules->{$node->{$self->_type_field}}) {
+            if ( defined $rules->{$node->{$self->_type_field}}->{$parent->{$self->_type_field}}) {
+                return $rules->{$node->{$self->_type_field}}->{$parent->{$self->_type_field}};
+            }
+            else { return 0 }
         }
         else { return 0 }
     }
@@ -308,6 +313,7 @@ sub _clone_node {
     return $clone;
 }
 
+# return key name of children parent
 sub _key_children {
     my ($self, $node, $parent) = @_;
 
@@ -335,7 +341,7 @@ sub _key_children {
     # parent haven't type
     else {
         if ( defined $node->{$self->_type_field}) {
-            die 'node have a type but not the parent !';
+            die "node " . $node->{$self->_search_field} . " [id:". $node->{id} . "] have a type but not the parent !";
         }
         else {
             $key_children = 'children';
@@ -543,13 +549,14 @@ sub add {
   $is_exist_in_backend = 1
       if $self->tree->{$node->{id}};
 
-  # save node in tree
+  # save ref node in tree
   $self->tree->{$node->{id}} = $node;
 
   my $key_children;
   my $update_children = 0;
-  # node == root
+  # node != root
   if ( defined $parent && $parent ) {
+
       $key_children = $self->_key_children($node, $parent);
 
       if ( $parent ) {
@@ -592,8 +599,12 @@ sub update {
   my ($self, $node, $datas) = @_;
 
   foreach my $k ( sort keys %$datas ) {
-      next if ( ! defined $node->{$k});
-      next if ( $node->{$k} eq $datas->{$k});
+
+      if ( ! defined $node->{$k} ){
+              $self->_log("update: can not update, node->{$k} is not defined but _force_create_key is not equal 1");
+              next;
+      }
+      elsif ( $node->{$k} eq $datas->{$k} ){next}
 
       my $previous = $node->{$k};
       my $parent   = $node->{parent};
@@ -610,7 +621,14 @@ sub update {
           $self->_update_children_position(@{$node->{parent}->{$key_children}})
               if ( defined $node->{$self->_position_field});
       }
+      elsif ( $k eq $self->_type_field) {
+          # check rules
+          return 0 if ! $self->_check_rules({ type => $datas->{$k} }, $node->{parent});
+      }
       elsif ( $k eq 'parent') {
+
+          return 0 if ! $self->_check_rules($node, $datas->{parent});
+
           my $old = $self->_remove_child_father($node);
           my $new_parent = $datas->{$k};
           push(@{$new_parent->{$key_children}}, $old);
